/****************************************************************************/
/**   vjbox  Kernel-Daemon                                                 **/
/**                                                                        **/
/** Copyright (C) 1997 Jochen Tillmann-Sarrazin, DG6VJ                     **/
/** All rights reserved.                                                   **/
/****************************************************************************/


#include "global.h"
#include "config.h"

int sock_fd;
int stn_fd;		/* fd zu der Gegenstation */


/* Wird aufgerufen, wenn das Hauptprogramm beendet wird. */
static void MainTerminate()
{
  close(sock_fd);
  
  /* Den Speicher fr die boards-Liste wieder freigeben */
  close_boards_box();
  
  /* Den Speicher fr die reject.box-Liste freigeben */
  close_reject_box();

  /* Den Speicher fr die server.box-Liste freigeben */
  close_server_box();
  
  _exit(0);
}



static void ChildWait()
{
  pid_t pid;
  int status;
  
  
  /* Signalverarbeitung fr das nchste Signal wieder aktivieren */
  signal(SIGCHLD,(void *)ChildWait);
  
  pid = wait(&status);
  
  close(stn_fd);

}






int main()
{
  struct sockaddr_in sock_addr;
  struct sockaddr_in sock_addr_new;
  fd_set fd_mask;
  int addrlen;
  int new_pid;



  /* Irgendwelche globalen Variablen initialisieren */
  init_globals();
  
  /* Konfig-File (vjboxd.conf) einlesen */
  read_vjboxd_conf();

  /* Einen FIFO erstellen, in den die einzelnen Instanzen die BIDs 'reinschreiben, die neu 'reingekommen
   * sind. */
/*  make_bidfifo();*/
  bidfifofd = -1;

  /* Das File boards.box einlesen. (Transfer- und Lifetime-Informationen) */
  read_boards_box();

  /* system/reject.box einlesen */
  read_reject_box();
  
  /* system/server.box einlesen */
  read_server_box();
  
  /* openlog()-Aufruf */
  init_syslogd();
  
  /* Verschiedene Verzeichnisse anlegen, wenn sie noch nicht vorhanden sind */
  make_directories();
  
  
  
  /* Socket initialisieren und empfangsfertig machen. */

  if ((sock_fd = socket(AF_INET,SOCK_STREAM,0)) == -1) {
    printf("socket() - failure\n");
    _exit(2);
  }



  sock_addr.sin_family = AF_INET;
  sock_addr.sin_port = htons(boxport);
  sock_addr.sin_addr.s_addr = htonl(INADDR_ANY);

  if (bind(sock_fd,(struct sockaddr *)&sock_addr,sizeof(sock_addr)) != 0) {
    printf("bind() - failure.\nPort %i seems to be busy - please wait a little and try again.\n",boxport);
    _exit(3);
  }

  if (listen(sock_fd,MAXLISTEN) != 0) {
    printf("listen() - failure\n");
    exit(4);
  }
  

  if ((clktck = sysconf(_SC_CLK_TCK)) < 0) {
    printf("sysconf(_SC_CLK_TCK) - failure\n");
    _exit(5);
  }

  
  
  /* Signalhandler installieren, damit der Daemon auch korrekt beendet werden kann */
  signal(SIGTERM,(void *)MainTerminate);
  
  /* Wenn ein Kindprozess terminiert nach ChildWait springen, da sonst
   * ein Zombie entstehen kann.
   */
  signal(SIGCHLD,(void *)ChildWait);
  
  
  /* Jetzt einen fork() ausfhren, damit der Daemon im Hintergrund arbeitet, und nicht
   * eine Konsole belegt.
   */
  if (fork() != 0) {
    printf("vjbox V%s (%s)  Copyright (C) 1997 Jochen Sarrazin, DG6VJ\nAll rights reserved.\n",
            VERSION,DATUM);
    printf("vjbox bound to port %i with call %s%s\n",boxport,mycall,myhierarch);
    _exit(0);
  }
  setsid();
  umask(0);
/*  if (fork() != 0) _exit(0);
*/
  
  
  /* Sooo... Jetzt hren wir also auf dem angegebenen Port. Jetzt wird
   * geguckt, ob davon was kommt - und zwar mit select(). */
  
  /* grosse Hauptschleife */
  for(;;) {
    
    FD_ZERO(&fd_mask);
    FD_SET(sock_fd,&fd_mask);
    
    select(sock_fd+1,&fd_mask,NULL,NULL,NULL);
    
    /* Jup, da will uns jemand connecten */
    if (FD_ISSET(sock_fd,&fd_mask)) {
      /* Wir wollen den connect */
      addrlen = sizeof(sock_addr_new);
      stn_fd = accept(sock_fd,(struct sockaddr *)&sock_addr_new,&addrlen);

      /* Einen neuen Prozess fr diesen Login erzeugen */
      
      new_pid = fork();
      
      if (new_pid == 0) {
        /* Wir sind im Kindprozess */
        
        /* Das PID-File fr diesen Prozess im ./userdata - Verzeichnis erzeugen */
        create_pidfile();
        
        child_fd = stn_fd;
        hauptschleife();
        
      }
      else {
        /* Noch im Elternprozess */
        
        /* fd zur Gegenstation hier schlieen, da sie nur im Kindprozess bentigt wird. */
        close(stn_fd);
      }
    }
  }
}













